"""Exceptions triggered to warn about internal errors in Assonant Data class module."""


class AssonantDataClassesError(Exception):
    """Custom Exception to warn when any error occurs inside Assonant Data Classes Modules."""
